# -*- coding: utf-8 -*-
from django import forms

from django.forms.widgets import Textarea
from django.forms.widgets import SelectDateWidget

from gestion.fields import AddressWidget
from .models import DonneeFormulaire, MODELES_CHAMP, get_filieres, Inscription, CHAMPS_FORM
from datetime import date

annee = date.today().year
choix_promo = [(str(i), str(i)) for i in range(annee, annee + 6)]


class InscriptionForm(forms.ModelForm):
    def __init__(self, ecole, filiere_init, *args, **kwargs):
        super(InscriptionForm, self).__init__(*args, **kwargs)

        instance = kwargs.get("instance", None)

        if instance:
            for type_champ in MODELES_CHAMP.values():
                for champ in type_champ.objects.filter(ecole=ecole):
                    try:
                        valeur = DonneeFormulaire.objects.get(inscription=instance, ident_champ=champ.ident).valeur
                    except DonneeFormulaire.DoesNotExist:
                        valeur = None
                    champ.ajouter_a(self, valeur)
        else:
            for type_champ in MODELES_CHAMP.values():
                for champ in type_champ.objects.filter(ecole=ecole):
                    champ.ajouter_a(self, None)

        filieres = get_filieres(ecole)
        filieres = (("", "----------"),) + tuple(filieres)

        self.fields["filiere"] = forms.ChoiceField(
            required=True, label="Filière", initial=filiere_init, choices=filieres
        )

        self.fields["promo"] = forms.ChoiceField(
            required=True, label="Promo", initial=str(annee + 3), choices=choix_promo
        )

    def clean_nom(self):
        nom = self.cleaned_data["nom"].strip()

        if nom.lower() == nom or (len(nom) > 1 and nom.upper() == nom):
            raise forms.ValidationError(
                "Capitalisation incorrecte du nom : "
                "Il ne doit pas être entièrement en majuscules ou en "
                "minuscules."
            )

        return nom

    def clean_prenom(self):
        prenom = self.cleaned_data["prenom"].strip()

        if prenom.lower() == prenom or (len(prenom) > 1 and prenom.upper() == prenom):
            raise forms.ValidationError(
                "Capitalisation incorrecte du "
                "prénom : Il ne doit pas être entièrement en majuscules ou en "
                "minuscules."
            )

        return prenom

    class Meta:
        model = Inscription
        exclude = ()
        widgets = {
            "adresse": AddressWidget,
            "adr_parents": Textarea(attrs={"rows": 4}),
            "date_naissance": SelectDateWidget(years=range(annee - 30, annee - 10)),
        }


class TypeChampForm(forms.Form):
    type_champ = forms.ChoiceField(choices=CHAMPS_FORM, label="Type de champ")


class ReglagesChampForm(forms.Form):
    type_champ = forms.ChoiceField(choices=CHAMPS_FORM, label="Type de champ")


class ModelFormGeneriqueOptions(object):
    def __init__(self, model=None, fields=None, exclude=None, widgets=None):
        self.model = model
        self.fields = fields
        self.exclude = exclude
        self.widgets = widgets


class ModelFormGenerique(forms.BaseModelForm):
    def __init__(self, modele, *args, **kwargs):
        opts = self._meta = ModelFormGeneriqueOptions(model=modele)
        self.declared_fields = None
        self.base_fields = forms.models.fields_for_model(
            opts.model,
            opts.fields,
            opts.exclude,
            opts.widgets,
            lambda f, **kwargs: f.formfield(**kwargs),
        )
        super(ModelFormGenerique, self).__init__(*args, **kwargs)
